`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    01:03:18 04/25/2016 
// Design Name: 
// Module Name:    lab08_1 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module lab08_1(
   lap_or_rst,
	stop_or_start,
   clk,
	ftsd_ctl,
	display
    );

	//Inputs
	input lap_or_rst;
	input stop_or_start;
	input clk;
	//Output
	output [14:0] display; // 14 segment display control
	output [3:0] ftsd_ctl; // scan control for 14-segment display
	//wire
	wire [1:0] ftsd_ctl_en; // divided output for ssd scan control
	wire r_t; //long press from lap_or_reset
	wire clk_25, clk_21, clk_18;
	wire lap_rst_debounced, stop_or_start_debounced;
	wire lap_rst_dealed, stop_or_start_dealed;
	wire [3:0] minute1, minute0, second1, second0; //counter output
	wire [3:0] ftsd_in, dis3, dis2, dis1, dis0;
	wire count_enable;
	

	assign rst_n = ~r_t;
	
	//***********************************//
	//DC, OP
	//***********************************//
	//For lap_or_reset
	debounce_circuit U0(  //debounce for lap
	  .clk(clk_18), // clock control
	  .rst_n(rst_n), // reset
	  .pb_in(lap_or_rst), //push button input
	  .pb_debounced(lap_rst_debounced) // debounced push button output
	);
	debounce_circuit U1(  //debounce for reset
	  .clk(clk_25), // clock control
	  .rst_n(rst_n), // reset
	  .pb_in(lap_or_rst), //push button input
	  .pb_debounced(r_t) // debounced push button output
	);
	one_pulse U2(
	  .clk(clk),  // clock input
	  .rst_n(rst_n), //active low reset
	  .in_trig(lap_rst_debounced), // input trigger
	  .out_pulse(lap_rst_dealed) // output one pulse 
	);
	
	//For stop_or_start
	debounce_circuit U3(
	  .clk(clk_18), // clock control
	  .rst_n(rst_n), // reset
	  .pb_in(stop_or_start), //push button input
	  .pb_debounced(stop_or_start_debounced) // debounced push button output
	);
	one_pulse U4(
	  .clk(clk),
	  .rst_n(rst_n),
	  .in_trig(stop_or_start_debounced),
	  .out_pulse(stop_or_start_dealed) 
	);


	//***********************************//
	//Stopwatch
	//***********************************//
	min_sec_counter U5(
	  .minute1(minute1),
	  .minute0(minute0),
	  .second1(second1),
	  .second0(second0),
	  .clk(clk_25),
	  .rst_n(rst_n),
	  .count_enable(count_enable)
	);

	//***********************************//
	//Deal with the display
	//***********************************//
	timer_lap_fsm U6(
	  .minute1(minute1),
	  .minute0(minute0),
	  .second1(second1),
	  .second0(second0),
	  .showout3(dis3),
	  .showout2(dis2),
	  .showout1(dis1),
	  .showout0(dis0),
	  .clk(clk),
	  .rst_n(rst_n),
	  .control(lap_rst_dealed)
	);
	
	start_stop_fsm U10(
		.clk(clk),
		.rst_n(rst_n),
		.in(stop_or_start_dealed), //fsm input
		.count_enable(count_enable)
	);
	
	//***********************************//
	//Display on ftsd
	//***********************************//
	// frequency divider 1/(2^25)
	freq_div U7(
	  .clk_25(clk_25),
	  .clk_21(clk_21),
	  .clk_18(clk_18),
	  .clk_ctl(ftsd_ctl_en), // divided clock for scan control 
	  .clk(clk), // clock from the crystal
	  .rst_n(rst_n) // reset
	);
	// Scan control
	scan_ctl U8(
	  .ftsd_ctl(ftsd_ctl), // ftsd display control signal 
	  .ftsd_in(ftsd_in), // output to ftsd display
	  .in0(dis3), // 1st input
	  .in1(dis2), // 2nd input
	  .in2(dis1), // 3rd input
	  .in3(dis0),  // 4th input
	  .ftsd_ctl_en(ftsd_ctl_en) // divided clock for scan control
	);

	// binary to 14-segment display decoder
	ftsd U9(
	  .display(display), // 14-segment display output
	  .in(ftsd_in)  // BCD number input
	);
	
endmodule
