`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    20:13:41 04/25/2016 
// Design Name: 
// Module Name:    min_sec_counter 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
`define ENABLED 1 // ENABLE indicator
`define DISABLED 0 // EIDABLE indicator
module min_sec_counter(
   minute1,
	minute0,
	second1,
	second0,
	clk, // global clock
	rst_n, //active low reset
	count_enable
	);

	// outputs
	output [3:0] minute1;
	output [3:0] minute0;
	output [3:0] second1;
	output [3:0] second0;

	// inputs
	input clk; // global clock signal
	input rst_n; // low active reset
	input count_enable;

	// temporatory nets
	reg load_def_second, load_def_minute; // enabled to load default value
	wire cout_second0, cout_second1, cout_minute0, cout_minute1; // BCD counter carryout
	

	//**************************************//
	// Second 
	//**************************************//
	// return from 59 to 00
	always @(second0 or second1)
	  if ((second1==4'd5)&&(second0==4'd9))
		 load_def_second = `ENABLED;
	  else
		 load_def_second = `DISABLED;

	// counter for second0
	upcounter Usec0(
	  .value(second0),
	  .carry(cout_second0),  // carry out
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(count_enable),  // depending on fsm
	  .load_default(load_def_second),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);

	// counter for second1
	upcounter Usec1(
	  .value(second1),
	  .carry(cout_second1),  // carry out
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_second0),  // increasing when second0 carry out
	  .load_default(load_def_second),  // enable load default value
	  .def_value(4'd0) // default value for counter 
	);

	//**************************************//
	// Minute
	//**************************************//
	// return from 59 to 00
	always @(second0 or second1)
	  if ((minute1==4'd5)&&(4'd9)&&load_def_second)
		 load_def_minute = `ENABLED;
	  else
		 load_def_minute = `DISABLED;

	// counter for minute0
	upcounter Umin0(
	  .value(minute0),
	  .carry(cout_minute0),  // carry out
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_second),  // increasing when second carry out
	  .load_default(load_def_minute),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);

	// counter for minute0
	upcounter Umin1(
	  .value(minute1),
	  .carry(cout_minute1),  // carry out
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_minute0),  //  increasing when minute carry out
	  .load_default(load_def_minute),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);



endmodule
